"""This Module allows for the import of passwords"""

from __future__ import print_function
import getpass
import os.path
import yaml
import libpkpass.crypto as crypto
from libpkpass.commands.command import Command
from libpkpass.password import PasswordEntry
from libpkpass.errors import CliArgumentError, LegacyImportFormatError, FileOpenError


    ####################################################################
class Import(Command):
    """This Class implements the cli functionality of import"""
    ####################################################################
    name = 'import'
    description = 'Import passwords that you have saved to a file'
    selected_args = Command.selected_args + ['pwfile', 'stdin', 'nopassphrase', 'dstpwstore',
                                             'card_slot', 'nocrypto',]

        ####################################################################
    def _run_command_execution(self):
        """ Run function for class.                                      """
        ####################################################################
        try:
            contents = ""
            with open(self.args['pwfile'], 'r') as fcontents:
                contents = fcontents.read().strip()
            if self.args['nocrypto']:
                self._file_handler(contents)
            else:
                passwd = getpass.getpass("Please enter the password for the file: ")
                passwords = []
                for password in contents.split("\n"):
                    passwords.append(crypto.sk_decrypt_string(password, passwd))
                self._file_handler("\n".join(passwords))
        except IOError:
            raise FileOpenError(self.args['pwfile'], "No such file or directory")

        ####################################################################
    def _file_handler(self, string):
        """This function handles the contents of a file"""
        ####################################################################
        try:
            self._yaml_file(yaml.safe_load(string))
        except TypeError:
            try:
                self._flat_file(string.strip().split("\n"))
            except TypeError:
                raise LegacyImportFormatError

        ####################################################################
    def _flat_file(self, passwords):
        """This function handles the simple key:value pair"""
        ####################################################################
        print("INFO: Flat password file detected, using 'imported' as description \
you can manually change the description in the file if you would like")
        db_len = len(passwords)
        i = 1
        for password in passwords:
            psplit = password.split(":")
            fname = psplit[0].strip()
            pvalue = psplit[1].strip()
            self.args['pwname'] = fname
            self.create_or_update_pass(pvalue, "imported", self.args['identity'])
            self.progress_bar(i, db_len)
            i += 1
        print("")

        ####################################################################
    def _yaml_file(self, passwords):
        """This function handles the yaml format of pkpass"""
        ####################################################################
        myidentity = self.identities.iddb[self.args['identity']]
        uid = myidentity['uid']
        pwstore = self.args['pwstore']
        db_len = len(passwords)
        i = 1
        for password in passwords:
            plaintext_str = passwords[password]['recipients'][uid]['encrypted_secret']
            full_path = os.path.join(pwstore, password)
            self.args['pwname'] = password
            self.args['overwrite'] = True
            plist = list(passwords[password]['recipients'])
            if os.path.isfile(full_path):
                existing_password = PasswordEntry()
                existing_password.read_password_data(full_path)
                self.args['overwrite'] = False
                passwords[password]['metadata'] = existing_password['metadata']
                plist += list(existing_password['recipients'])

            description = passwords[password]['metadata']['description']
            authorizer = passwords[password]['metadata']['authorizer']
            self.create_or_update_pass(plaintext_str, description, authorizer, plist)
            self.progress_bar(i, db_len)
            i += 1
        print("")

        ####################################################################
    def _validate_args(self):
        """ Ensure arguments are appropriate for this command           """
        ####################################################################
        for argument in ['pwfile', 'keypath']:
            if argument not in self.args or self.args[argument] is None:
                raise CliArgumentError(
                    "'%s' is a required argument" % argument)



#!/usr/bin/env python
"""This Module tests crypto functionality"""

from __future__ import print_function
import unittest
import libpkpass.crypto as crypto
from libpkpass.identities import IdentityDB


class TestBasicFunction(unittest.TestCase):
    """This class tests the crypto class"""
    def setUp(self):
        self.plaintext = "PLAINTEXT"

        self.certdir = 'test/pki/intermediate/certs'
        self.keydir = 'test/pki/intermediate/private'
        self.cabundle = 'test/pki/intermediate/certs/ca-bundle'

        self.identities = IdentityDB()
        self.identities.load_certs_from_directory(self.certdir, self.cabundle)
        self.identities.load_keys_from_directory(self.keydir)

    # Encrypt strings for all test identities and make sure they are different
    def test_encrypt_string(self):
        """Test encrypting a string"""
        results = []
        for _, identity in self.identities.iddb.items():
            results.append(crypto.pk_encrypt_string(self.plaintext, identity))
        self.assertTrue(results[0] != results[1])

    # Encrypt/Decrypt strings for all test identities and make sure we get back what we put in
        for _, identity in self.identities.iddb.items():
            (ciphertext, derived_key) = crypto.pk_encrypt_string(
                self.plaintext, identity)
            plaintext = crypto.pk_decrypt_string(
                ciphertext, derived_key, identity, None)
            self.assertEqual(self.plaintext, plaintext)

    def test_verify_string(self):
        """verify string is correct"""
        results = []
        for _, identity in self.identities.iddb.items():
            results.append(crypto.pk_sign_string(
                self.plaintext, identity, None))
        self.assertTrue(results[0] != results[1])

        for _, identity in self.identities.iddb.items():
            signature = crypto.pk_sign_string(self.plaintext, identity, None)
            self.assertTrue(crypto.pk_verify_signature(
                self.plaintext, signature, identity))

    def test_cert_fingerprint(self):
        """Verify fingerprint is correct"""
        for _, identity in self.identities.iddb.items():
            fingerprint = crypto.get_cert_fingerprint(identity)
            self.assertTrue(len(fingerprint.split(':')) == 20)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""This module tests the rename module"""

from __future__ import absolute_import
import unittest
import argparse
import mock
import libpkpass.commands.cli as cli
import libpkpass.commands.rename as rename
from libpkpass.errors import DecryptionError, CliArgumentError
from .basetest.basetest import CONFIG, BADPIN

class RenameTests(unittest.TestCase):
    """This class tests the rename class"""

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='rename', identity='r1',
                                                nopassphrase="true",
                                                all=None,
                                                pwname='test',
                                                rename='retest',
                                                config=CONFIG))
    def test_safe_error(self, subparser_name):
        """test decryption functionality"""
        ret = False
        try:
            rename.Rename(cli.Cli())
        except DecryptionError:
            ret = True
        self.assertTrue(ret)

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='rename', identity='bleh',
                                                nopassphrase="true",
                                                all=None,
                                                pwname='test',
                                                rename='retest',
                                                config=CONFIG))
    def test_recipient_not_in_database(self, subparser_name):
        """test decryption functionality"""
        ret = False
        try:
            rename.Rename(cli.Cli())
        except CliArgumentError as error:
            if error.msg == "Error: Your user 'bleh' is not in the recipient database":
                ret = True
        self.assertTrue(ret)

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='rename', identity='r1',
                                                nopassphrase="true",
                                                all=True,
                                                pwname='test',
                                                rename='retest',
                                                overwrite="true",
                                                config=CONFIG))
    def test_rename_decryption_error(self, subparser_name):
        """test decryption functionality"""
        ret = False
        try:
            rename.Rename(cli.Cli())
        except DecryptionError as error:
            if error.msg == BADPIN:
                ret = True
        self.assertTrue(ret)

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='rename', identity='r1',
                                                nopassphrase="true",
                                                all=True,
                                                pwname='test',
                                                overwrite="true",
                                                config=CONFIG))
    def test_rename_cli_error(self, subparser_name):
        """test decryption functionality"""
        ret = False
        try:
            rename.Rename(cli.Cli())
        except CliArgumentError as error:
            if error.msg == "'rename' is a required argument":
                ret = True
        self.assertTrue(ret)

if __name__ == '__main__':
    unittest.main()



"""This Module defines what a passworddb should look like"""
import os
import yaml
from libpkpass.password import PasswordEntry
from libpkpass.errors import PasswordIOError

    ##############################################################################
class PasswordDB(object):
    """ Password database object.  Gets and retrieves password entries from places
        passwords are stored"""
    ##############################################################################

    def __init__(self, mode='Filesystem'):
        self.mode = mode
        self.pwdb = {}

    def __repr__(self):
        return "%s(%r)" % (self.__class__, self.__dict__)

    def __str__(self):
        return "%r" % self.__dict__

    def __sizeof__(self):
        return len(self.pwdb)

    ##############################################################################
    def load_password_data(self, password_id):
        """ Load and return password from wherever it may be stored"""
    ##############################################################################
        if not password_id in self.pwdb.keys() and self.mode == 'Filesystem':
            self.pwdb[password_id] = self.read_password_data_from_file(password_id)
        return self.pwdb[password_id]

    #############################################################################
    def load_from_directory(self, pwstore):
        """ Load all passwords from directory """
    #############################################################################
        for path, _, files in os.walk(pwstore):
            for passwordname in files:
                passwordpath = os.path.join(path, passwordname)
                self.load_password_data(passwordpath)

    ##############################################################################
    def save_password_data(self, password_id, overwrite=False):
        """ Store a password to wherever it may be stored """
    ##############################################################################
        if self.mode == 'Filesystem':
            self.write_password_data_to_file(
                self.pwdb[password_id], password_id, overwrite)

    ##############################################################################
    def read_password_data_from_file(self, filename):
        """ Open a password file, load passwords and read metadata """
    ##############################################################################
        try:
            with open(filename, 'r') as fname:
                password_data = yaml.safe_load(fname)
                password_entry = PasswordEntry()
                password_entry.metadata = password_data['metadata']
                password_entry.recipients = password_data['recipients']
                if 'escrow' in password_data:
                    password_entry.escrow = password_data['escrow']
            password_entry.validate()
            return password_entry
        except (OSError, IOError, TypeError):
            raise PasswordIOError("Error reading '%s' perhaps a path error for the db, or malformed file" % filename)

    #############################################################################
    def write_password_data_to_file(self, password_data, filename, overwrite=False):
        """ Write password data and metadata to the appropriate password file """
    ##############################################################################
        try:
            if not os.path.isdir(os.path.dirname(filename)):
                os.makedirs(os.path.dirname(filename))
            with open(filename, 'w+') as fname:
                passdata = {key: value for key, value in password_data.todict().items() if value}
                fname.write(yaml.dump(passdata,
                                      default_flow_style=False))
        except (OSError, IOError):
            raise PasswordIOError("Error creating '%s'" % filename)



"""This module handles the CLI for password recovery"""

from __future__ import print_function
from builtins import input
from libpkpass.escrow import pk_recover_secret
from libpkpass.commands.command import Command

    ####################################################################
class Recover(Command):
    """This class implements the CLI functionality of recovery for passwords"""
    ####################################################################
    name = 'recover'
    description = 'Recover a password that has been distributed using escrow functions'
    selected_args = Command.selected_args + ['pwstore', 'keypath', 'nosign', 'escrow_users', 'min_escrow']

        ####################################################################
    def _run_command_execution(self):
        """ Run function for class.                                      """
        ####################################################################
        print("If the password returned is not correct, you may need more shares")
        shares = input("Enter comma separated list of shares: ")
        shares = shares.split(",")
        shares = map(str.strip, shares)
        print(pk_recover_secret(shares))

        ####################################################################
    def _validate_args(self):
        ####################################################################
        pass

        ####################################################################
    def _validate_combinatorial_args(self):
        ####################################################################
        pass



#!/usr/bin/env python
"""This module tests the list module"""

from __future__ import absolute_import
import unittest
import argparse
import mock
import libpkpass.commands.cli as cli
import libpkpass.commands.list as pklist
from libpkpass.errors import CliArgumentError
from .basetest.basetest import CONFIG, captured_output

PASSWORD_LIST_0 = "Passwordsfor'r2':"
PASSWORD_LIST_1 = "Passwordsfor'r1':test/passwords/test:Distributor:ginsburgnmName:test"

class ListTests(unittest.TestCase):
    """This class tests the list class"""


    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='list', identity='bleh',
                                                nopassphrase="true",
                                                config=CONFIG,
                                                color="false"))
    def test_recipient_not_in_database(self, subparser_name):
        """test bad recipient functionality"""
        ret = False
        try:
            pklist.List(cli.Cli())
        except CliArgumentError as error:
            if error.msg == "Error: Your user 'bleh' is not in the recipient database":
                ret = True
        self.assertTrue(ret)

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='list', identity='r2',
                                                nopassphrase="true",
                                                config=CONFIG,
                                                color="false"))
    def test_list_none(self, subparser_name):
        """test list functionality"""
        ret = False
        with captured_output() as (out, _):
            pklist.List(cli.Cli())
        output = "".join(out.getvalue().strip().split())
        if output == PASSWORD_LIST_0:
            ret = True
        self.assertTrue(ret)


    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='list', identity='r1',
                                                nopassphrase="true",
                                                config=CONFIG,
                                                color="false"))
    def test_list_one(self, subparser_name):
        """test list functionality"""
        ret = False
        with captured_output() as (out, _):
            pklist.List(cli.Cli())
        output = "".join(out.getvalue().strip().split())
        if output == PASSWORD_LIST_1:
            ret = True
        self.assertTrue(ret)

if __name__ == '__main__':
    unittest.main()



"""This module allows for the updating of passwords"""

from __future__ import print_function
import getpass
import os
from builtins import input
import libpkpass.util as util
from libpkpass.password import PasswordEntry
from libpkpass.commands.command import Command
from libpkpass.errors import CliArgumentError, PasswordMismatchError, NotThePasswordOwnerError,\
        BlankPasswordError

    ####################################################################
class Update(Command):
    """This class implements the CLI functionality of updating existing passwords"""
    ####################################################################
    name = 'update'
    description = 'Change a password value and redistribute to recipients'
    selected_args = Command.selected_args + ['pwname', 'pwstore', 'overwrite', 'stdin', 'keypath',
                                             'nopassphrase', 'nosign', 'card_slot', 'escrow_users',
                                             'min_escrow', 'noescrow']

        ####################################################################
    def _run_command_execution(self):
        """ Run function for class.                                      """
        ####################################################################
        password = PasswordEntry()
        password.read_password_data(os.path.join(
            self.args['pwstore'], self.args['pwname']))
        safe, owner = self.safety_check()
        if safe or self.args['overwrite']:
            self.recipient_list = password['recipients'].keys()
            print("The following list of users are the current distribution list:")
            print(", ".join(util.sort(self.recipient_list)))
            correct_distribution = input("Is this list correct? (y/N) ")
            if not correct_distribution or correct_distribution.lower()[0] == 'n':
                self._new_distribution()

            self._validate_identities(self.recipient_list)

            password1 = getpass.getpass("Enter updated password: ")
            if password1.strip() == "":
                raise BlankPasswordError
            password2 = getpass.getpass("Enter updated password again: ")
            if password1 != password2:
                raise PasswordMismatchError

            # This is due to a poor naming convention; we don't want to go through the
            # create or update, because we are not updating a single record in the yaml
            # we are burning it to the ground and redoing the entire thing
            self.create_pass(password1,
                             password['metadata']['description'],
                             password['metadata']['authorizer'],
                             self.recipient_list)
        else:
            raise NotThePasswordOwnerError(self.args['identity'], owner, self.args['pwname'])

        ####################################################################
    def _new_distribution(self):
        """enter a new recipient list"""
        ####################################################################
        breaker = False
        while not breaker:
            self.recipient_list = input("Please enter a comma delimited list: ")
            self.recipient_list = list(set(self._convert_strings_to_list(self.recipient_list)))
            print(self.recipient_list)
            try:
                self._validate_identities(self.recipient_list)
                breaker = True
            except CliArgumentError as err:
                print(err)
                continue

        ####################################################################
    def _validate_args(self):
        ####################################################################
        for argument in ['pwname', 'keypath']:
            if argument not in self.args or self.args[argument] is None:
                raise CliArgumentError(
                    "'%s' is a required argument" % argument)



"""This Modules allows for distributing created passwords to other users"""
from __future__ import print_function
import fnmatch
import os
from builtins import input
import libpkpass.util as util
from libpkpass.commands.command import Command
from libpkpass.passworddb import PasswordDB
from libpkpass.password import PasswordEntry
from libpkpass.errors import CliArgumentError


    ####################################################################
class Distribute(Command):
    """This Class implements the CLI functionality for ditribution"""
    ####################################################################
    name = 'distribute'
    description = 'Distribute existing password entry/ies to another entity [matching uses python fnmatch]'
    selected_args = Command.selected_args + ['pwname', 'pwstore', 'users', 'groups', 'stdin',
                                             'min_escrow', 'escrow_users', 'keypath', 'nopassphrase',
                                             'nosign', 'card_slot', 'noescrow']

        ####################################################################
    def _run_command_execution(self):
        """ Run function for class.                                      """
        ####################################################################
        passworddb = PasswordDB()
        passworddb.load_from_directory(self.args['pwstore'])
        filtered_pdb = util.dictionary_filter(
            os.path.join(self.args['pwstore'], self.args['pwname']),
            passworddb.pwdb,
            [self.args['identity'], 'recipients']
        )
        print("The following password files have matched:")
        print(*filtered_pdb.keys(), sep="\n")
        correct_distribution = input("Is this list correct? (y/N) ")
        if correct_distribution and correct_distribution.lower()[0] == 'y':
            passworddb.pwdb = filtered_pdb
            db_len = len(passworddb.pwdb.keys())
            i = 0
            self.progress_bar(i, db_len)
            for dist_pass, _ in passworddb.pwdb.items():
                password = PasswordEntry()
                password.read_password_data(dist_pass)
                if self.args['identity'] in password.recipients.keys():
                    # we shouldn't modify escrow on distribute
                    self.args['min_escrow'] = None
                    self.args['escrow_users'] = None
                    plaintext_pw = password.decrypt_entry(
                        self.identities.iddb[self.args['identity']],
                        passphrase=self.passphrase,
                        card_slot=self.args['card_slot'])

                    password.add_recipients(secret=plaintext_pw,
                                            distributor=self.args['identity'],
                                            recipients=self.recipient_list,
                                            identitydb=self.identities,
                                            passphrase=self.passphrase,
                                            card_slot=self.args['card_slot'],
                                            pwstore=self.args['pwstore']
                                           )

                    password.write_password_data(dist_pass)
                    i += 1
                    self.progress_bar(i, db_len)
            # format the progress bar appropriately after the loop
            print("")
        else:
            print("Exiting due to wrong password list")

        ####################################################################
    def _validate_args(self):
        ####################################################################
        for argument in ['pwname', 'keypath']:
            if argument not in self.args or self.args[argument] is None:
                raise CliArgumentError(
                    "'%s' is a required argument" % argument)



#!/usr/bin/env python
"""This Module tests iddb module"""

import unittest
import os.path
from libpkpass.identities import IdentityDB


class TestBasicFunction(unittest.TestCase):
    """This class tests the iddb class"""
    def setUp(self):
        self.certdir = 'test/pki/intermediate/certs'
        self.keydir = 'test/pki/intermediate/private'
        self.cabundle = 'test/pki/intermediate/certs/ca-bundle'

    def test_certificate_loading(self):
        """Load a cert into our test"""
        idobj = IdentityDB()
        idobj.load_certs_from_directory(self.certdir, self.cabundle)
        for identity in ('r1', 'r2', 'r3'):
            assert identity in idobj.iddb
            assert idobj.iddb[identity]['uid'] == identity
            assert os.path.isfile(idobj.iddb[identity]['certificate_path'])

    def test_key_loading(self):
        """Load a key into our test"""
        idobj = IdentityDB()
        idobj.load_keys_from_directory(self.keydir)
        for identity in ('r1', 'r2', 'r3'):
            assert identity in idobj.iddb
            assert idobj.iddb[identity]['uid'] == identity
            assert os.path.isfile(idobj.iddb[identity]['key_path'])

    def test_keycert_loading(self):
        """Load a keycert into our test"""
        idobj = IdentityDB()
        idobj.load_certs_from_directory(self.certdir, self.cabundle)
        idobj.load_keys_from_directory(self.keydir)
        for identity in ('r1', 'r2', 'r3'):
            assert identity in idobj.iddb
            assert idobj.iddb[identity]['uid'] == identity
            assert os.path.isfile(idobj.iddb[identity]['certificate_path'])
            assert os.path.isfile(idobj.iddb[identity]['key_path'])


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""This module tests the distribute module"""

from __future__ import absolute_import
import unittest
import argparse
import mock
import libpkpass.commands.cli as cli
import libpkpass.commands.distribute as distribute
from libpkpass.errors import DecryptionError, CliArgumentError
from .basetest.basetest import CONFIG, BADPIN

class DistributeTests(unittest.TestCase):
    """This class tests the distribute class"""


    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='distribute', identity='bleh',
                                                nopassphrase="true",
                                                pwname='test',
                                                config=CONFIG))
    def test_recipient_not_in_database(self, subparser_name):
        """test decryption functionality"""
        ret = False
        try:
            distribute.Distribute(cli.Cli())
        except CliArgumentError as error:
            if error.msg == "Error: Your user 'bleh' is not in the recipient database":
                ret = True
        self.assertTrue(ret)

    ##################
    # Temporarily commenting this test, it doesn't do alot and halts
    # automated testing after addition of filename globbing for distribute
    ##################
    # @mock.patch('argparse.ArgumentParser.parse_args',
    #             return_value=argparse.Namespace(subparser_name='distribute', identity='r1',
    #                                             nopassphrase="true",
    #                                             pwname='test',
    #                                             config=CONFIG))
    # def test_distribute_decrypt_error(self, subparser_name):
    #     """test decryption functionality"""
    #     ret = False
    #     try:
    #         distribute.Distribute(cli.Cli())
    #     except DecryptionError as error:
    #         if error.msg == BADPIN:
    #             ret = True
    #     self.assertTrue(ret)

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='distribute', identity='r1',
                                                nopassphrase="true",
                                                pwname=None,
                                                config=CONFIG))
    def test_distribute_cli_error(self, subparser_name):
        """test decryption functionality"""
        ret = False
        try:
            distribute.Distribute(cli.Cli())
        except CliArgumentError as error:
            if error.msg == "'pwname' is a required argument":
                ret = True
        self.assertTrue(ret)

if __name__ == '__main__':
    unittest.main()



"""This Module defines what a password contains"""

from __future__ import print_function
import uuid
import time
import os
import yaml
from libpkpass.escrow import pk_split_secret
from libpkpass.errors import NotARecipientError, DecryptionError, PasswordIOError, YamlFormatError
import libpkpass.crypto as crypto

    #######################################################################
class PasswordEntry(object):
    """ Password entry object.  Contains information about a password and
    related metadata, as well as a list of individually encrypted strings"""
    ##########################################################################

    def __init__(self, **kwargs):

        self.metadata = {'name': None,
                         'description': None,
                         'creator': None,
                         'authorizer': None,
                         'signature': None,
                         'schemaVersion': 'v1'}

        self.metadata.update(kwargs)
        self.recipients = {}
        self.escrow = {}

        ####################################################################
    def __getitem__(self, key):
        """Grab items like with dictionaries"""
        ####################################################################
        return getattr(self, key)

        ####################################################################
    def read_escrow(self, filename):
        """Read values from escrow section"""
        ####################################################################
        try:
            with open(filename, 'r') as fname:
                password_data = yaml.safe_load(fname)
                if 'escrow' in password_data.keys():
                    return password_data['escrow']
                return {}
        except (OSError, IOError, yaml.scanner.ScannerError, yaml.parser.ParserError):
            #we actually don't care here.. file might not exist yet
            return {}

        #######################################################################
    def process_escrow_map(
            self,
            escrow_map,
            split_secret=None,
            distributor=None,
            recipients=None,
            identitydb=None,
            encryption_algorithm='rsautl',
            passphrase=None,
            card_slot=None,
            escrow_users=None,
            minimum=None):
        """Process the escrow user map into escrow users"""
        #######################################################################
        escrow_guid = str(uuid.uuid1()).replace('-', '')
        if escrow_map:
            for key, value in escrow_map.items():
                if set(value['recipients']) == set(escrow_users):
                    escrow_guid = key
        self.escrow[escrow_guid] = {
            'metadata': {
                'minimum_escrow': minimum,
                'creator': distributor
                },
            'recipients':{}
            }
        i = 0
        for escrow_user in escrow_users:
            if escrow_user not in recipients:
                self.escrow[escrow_guid]['recipients'][escrow_user] = self._add_recipient(
                    escrow_user, split_secret[i], distributor,
                    identitydb, encryption_algorithm, passphrase, card_slot)
                i += 1


        #######################################################################
    def add_escrow(
            self,
            secret=None,
            escrow_users=None,
            minimum=None,
            pwstore=None):
        """ Add escrow users to the recipient list of this password object"""
        #######################################################################
        split_secret = pk_split_secret(secret, escrow_users, minimum)
        return (self.read_escrow(os.path.join(pwstore, self.metadata['name'])), split_secret)

        #######################################################################
    def add_recipients(
            self,
            secret=None,
            distributor=None,
            recipients=None,
            identitydb=None,
            encryption_algorithm='rsautl',
            passphrase=None,
            card_slot=None,
            escrow_users=None,
            minimum=None,
            pwstore=None):
        """ Add recipients to the recipient list of this password object"""
        #######################################################################
        if recipients is None:
            recipients = []
        for recipient in recipients:
            self.recipients[recipient] = self._add_recipient(recipient, secret, distributor,
                                                             identitydb, encryption_algorithm, passphrase,
                                                             card_slot)
        if escrow_users:
            #escrow_users may now be none after the set operations
            if (len(escrow_users) > 3) and (len(list((set(escrow_users) - set(recipients)))) < 3):
                print("warning: min_escrow requirement not met after removing password recipients from escrow user list")
                return
            escrow_map, split_secret = self.add_escrow(
                secret=secret,
                escrow_users=escrow_users,
                minimum=minimum,
                pwstore=pwstore)
            self.process_escrow_map(
                escrow_map,
                split_secret=split_secret,
                distributor=distributor,
                recipients=recipients,
                identitydb=identitydb,
                encryption_algorithm=encryption_algorithm,
                passphrase=passphrase,
                card_slot=card_slot,
                escrow_users=escrow_users,
                minimum=minimum)

        #######################################################################
    def _add_recipient(
            self,
            recipient,
            secret=None,
            distributor=None,
            identitydb=None,
            encryption_algorithm='rsautl',
            passphrase=None,
            card_slot=None):
        """Add recipient or sharer to list"""
        #######################################################################
        try:
            if encryption_algorithm == 'rsautl':
                (encrypted_secret, encrypted_derived_key) = crypto.pk_encrypt_string(
                    secret, identitydb.iddb[recipient])
            recipient_entry = {'encrypted_secret': encrypted_secret,
                               'derived_key': encrypted_derived_key,
                               'distributor': distributor,
                               'distributor_hash': identitydb.iddb[distributor]['subjecthash'],
                               'recipient_hash': identitydb.iddb[recipient]['subjecthash'],
                               # 'distributor_fingerprint': crypto.get_cert_fingerprint( identitydb.iddb[distributor] ),
                               # 'recipient_fingerprint': crypto.get_cert_fingerprint( identitydb.iddb[recipient] ),
                               'encryption_algorithm': encryption_algorithm,
                               'timestamp': time.time()}

            message = self._create_signable_string(recipient_entry)

            recipient_entry['signature'] = crypto.pk_sign_string(
                message,
                identitydb.iddb[distributor],
                passphrase, card_slot)

            return recipient_entry
        except KeyError:
            raise NotARecipientError(
                "Identity '%s' is not on the recipient list for password '%s'" %
                (recipient, self.metadata['name']))

        #######################################################################
    def decrypt_entry(self, identity=None, passphrase=None, card_slot=None):
        """ Decrypt this password entry for a particular identity
        (usually the user) """
        #######################################################################
        try:
            recipient_entry = self.recipients[identity['uid']]
        except KeyError:
            raise NotARecipientError(
                "Identity '%s' is not on the recipient list for password '%s'" %
                (identity['uid'], self.metadata['name']))

        try:
            return crypto.pk_decrypt_string(
                recipient_entry['encrypted_secret'],
                recipient_entry['derived_key'],
                identity,
                passphrase,
                card_slot)
        except DecryptionError:
            raise DecryptionError(
                "Error decrypting password named '%s'.  Perhaps a bad pin/passphrase?" %
                self.metadata['name'])

        #######################################################################
    def verify_entry(self, uid=None, iddb=None):
        """ Check to see if all signatures and user certificates are okay for
        the user accessing this password entry """
        #######################################################################
        identitydb = iddb.iddb
        recipient_entry = self.recipients[uid].copy()
        distributor = recipient_entry['distributor']
        iddb.verify_identity(distributor)
        signature = recipient_entry.pop('signature')
        message = self._create_signable_string(recipient_entry)
        sig_ok = crypto.pk_verify_signature(
            message, signature, identitydb[distributor])
        return {'distributor': distributor,
                'sigOK': sig_ok,
                'certOK': identitydb[distributor]['verified']}

        #######################################################################
    def _create_signable_string(self, recipient_entry):
        """ Helper function to create a string out of the information
        this PasswordEntry contains"""
        #######################################################################
        message = ""
        for key in sorted(recipient_entry.keys()):
            if key == 'timestamp':
                recipient_entry[key] = "{0:12.2f}".format(float(recipient_entry[key]))
            if key != 'signature':
                if isinstance(recipient_entry[key], bytes):
                    recipient_entry[key] = recipient_entry[key].decode('ASCII')
                message = message + str(key) + str(recipient_entry[key])
        return message

        #######################################################################
    def todict(self):
        """Returns a dictionary"""
        #######################################################################
        return vars(self)

        #######################################################################
    def validate(self):
        """This function validates a password object"""
        #######################################################################
        # for field in ['field1', 'field2']:
        #  if weird: raise PasswordValidationError(field, value)
        return True

        #######################################################################
    def __repr__(self):
        #######################################################################
        return "%s(%r)" % (self.__class__, self.__dict__)

        #######################################################################
    def __str__(self):
        #######################################################################
        return "%r" % self.__dict__

        ##########################################################################
    def read_password_data(self, filename):
        """ Open a password file, load passwords and read metadata"""
        ##########################################################################
        try:
            with open(filename, 'r') as fname:
                password_data = yaml.safe_load(fname)
                self.metadata = password_data['metadata']
                self.recipients = password_data['recipients']
                if 'escrow' in password_data:
                    self.escrow = password_data['escrow']
            self.validate()
        except (OSError, IOError) as error:
            raise PasswordIOError(
                "Error Opening %s due to %s" %
                (filename, error.strerror))
        except (yaml.scanner.ScannerError, yaml.parser.ParserError) as error:
            raise YamlFormatError(str(error.problem_mark), error.problem)

        ############################################################################################
    def write_password_data(self, filename, overwrite=False, encrypted_export=False, password=None):
        """Write password data to a password file """
        ############################################################################################
        self.validate()
        # check if the file is in the base password directory
        incwd = os.path.basename(filename) == filename
        # if not ensure path exists
        try:
            if not incwd and not os.path.isdir(os.path.dirname(filename)):
                os.makedirs(os.path.dirname(filename))
            passdata = {key: value for key, value in self.todict().items() if value}
            with open(filename, 'w') as fname:
                if encrypted_export:
                    encrypted = crypto.sk_encrypt_string(
                        yaml.safe_dump(passdata, default_flow_style=False),
                        password)
                    fname.write(encrypted + "\n")
                else:
                    fname.write(yaml.safe_dump(passdata, default_flow_style=False))
        except (OSError, IOError):
            raise PasswordIOError("Error creating '%s'" % filename)



#!/usr/bin/env python
"""This module provides base testing capabilites"""
import sys
from contextlib import contextmanager
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

global CONFIG
CONFIG = './test/.test_config'
global BADPIN
BADPIN = "Error decrypting password named 'test'.  Perhaps a bad pin/passphrase?"

@contextmanager
def captured_output():
    new_out, new_err = StringIO(), StringIO()
    old_out, old_err = sys.stdout, sys.stderr
    try:
        sys.stdout, sys.stderr = new_out, new_err
        yield sys.stdout, sys.stderr
    finally:
        sys.stdout, sys.stderr = old_out, old_err



"""This module allows for the creation of passwords"""
import getpass
from builtins import input
from libpkpass.commands.command import Command
from libpkpass.errors import CliArgumentError, PasswordMismatchError, BlankPasswordError


    ####################################################################
class Create(Command):
    """This class implements the CLI functionality of creation of passwords"""
    ####################################################################
    name = 'create'
    description = 'Create a new password entry and encrypt it for yourself'
    selected_args = Command.selected_args + ['pwname', 'pwstore', 'overwrite', 'stdin', 'keypath',
                                             'nopassphrase', 'nosign', 'card_slot', 'escrow_users',
                                             'min_escrow', 'noescrow']

        ####################################################################
    def _run_command_execution(self):
        """ Run function for class.                                      """
        ####################################################################
        password1 = getpass.getpass("Enter password to create: ")
        if password1.strip() == "":
            raise BlankPasswordError
        password2 = getpass.getpass("Enter password to create again: ")
        if password1 != password2:
            raise PasswordMismatchError

        description = input("Description: ")
        authorizer = input("Authorizer: ")
        self.create_or_update_pass(password1, description, authorizer)

        ####################################################################
    def _validate_args(self):
        ####################################################################
        for argument in ['pwname', 'keypath']:
            if argument not in self.args or self.args[argument] is None:
                raise CliArgumentError(
                    "'%s' is a required argument" % argument)



"""This module is a generic for all pkpass commands"""

from __future__ import print_function
import sys
import getpass
import json
import os
import yaml
from six import iteritems
from six import string_types
import libpkpass.util as util
from libpkpass.commands.arguments import ARGUMENTS as arguments
from libpkpass.password import PasswordEntry
from libpkpass.identities import IdentityDB
from libpkpass.crypto import print_card_info
from libpkpass.errors import NullRecipientError, CliArgumentError, FileOpenError, GroupDefinitionError,\
        PasswordIOError, JsonArgumentError, NotThePasswordOwnerError, ConfigParseError


    ##########################################################################
class Command(object):
    """ Base class for all commands.  Auotmatically registers with cli subparser
    and provides run execution for itself.                                     """
    ##########################################################################
    name = None
    description = None
    selected_args = ['verbosity', 'quiet', 'identity', 'cabundle', 'certpath', 'color', 'theme_map']
    passphrase = None

        ##################################################################
    def __init__(self, cli, iddb=None):
        """ Intialization function for class. Register with argparse   """
        ##################################################################
        self.cli = cli
        #default certpath to none because connect string is allowed
        self.args = {
            'ignore_decrypt': False,
            'identity': getpass.getuser(),
            'cabundle': './certs/ca-bundle',
            'keypath': './private',
            'pwstore': './passwords',
            'time': 10,
            'card_slot': None,
            'certpath': None,
            'escrow_users': None,
            'min_escrow': None,
            'noverify': None,
            'noescrow': False,
            'overwrite': False,
            'recovery': False,
            'rules': 'default',
            'theme_map': None,
            'color': True,
            'verbosity': 0,
            }
        self.recipient_list = []
        self.escrow_and_recipient_list = []
        self.iddbcached = iddb is not None
        self.identities = iddb if iddb else IdentityDB()
        cli.register(self, self.name, self.description)

        ##################################################################
    def register(self, parser):
        """ Registration function for class. Register with argparse      """
        ##################################################################
        for arg in sorted(self.selected_args):
            parser.add_argument(
                *arguments[arg]['args'],
                **arguments[arg]['kwargs'])

        ##################################################################
    def run(self, parsedargs):
        """ Passes the argparse Namespace object of parsed arguments   """
        ##################################################################
        self._run_command_setup(parsedargs)
        self._run_command_execution()

        ##################################################################
    def _convert_strings_to_list(self, argname):
        """ convert argparsed strings to lists for an argument """
        ##################################################################
        if argname in self.args:
            self.args[argname] = self.args[argname].split(",") if self.args[argname] else []
            self.args[argname] = [arg.strip() for arg in self.args[argname] if arg.strip()]
        elif isinstance(argname, string_types):
            argname = argname.split(",") if argname else []
            return [arg.strip() for arg in argname if arg.strip()]
        return None

        ##################################################################
    def _handle_boolean_args(self, argname):
        ##################################################################
        if isinstance(self.args[argname], string_types):
            return self.args[argname].upper() == 'TRUE'
        return self.args[argname]

        ##################################################################
    def _handle_filepath_args(self):
        """Does filepath expansion for config args"""
        ##################################################################
        file_path_args = ['cabundle', 'pwstore', 'certpath', 'keypath']
        for arg in file_path_args:
            if arg in self.args and self.args[arg]:
                self.args[arg] = os.path.expanduser(self.args[arg])
        if 'connect' in self.args and self.args['connect'] and \
                'base_directory' in self.args['connect'] and self.args['connect']['base_directory']:
            self.args['connect']['base_directory'] = os.path.expanduser(self.args['connect']['base_directory'])

        ##################################################################
    def _run_command_setup(self, parsedargs):
        """ Passes the argparse Namespace object of parsed arguments   """
        ##################################################################

        # Build a dict out of the argparse args Namespace object and a dict from any
        # configuration files and merge the two with cli taking priority
        cli_args = vars(parsedargs)

        config_args = self._get_config_args(cli_args['config'], cli_args)
        self.args.update(config_args)

        connectmap = self._parse_json_arguments('connect')
        self._handle_filepath_args()

        fles = ['cabundle', 'pwstore']
        for key, value in iteritems(cli_args):
            if value is not None or key not in self.args:
                self.args[key] = value
            if key in fles and not os.path.exists(self.args[key]):
                raise FileOpenError(self.args[key], "No such file or directory")

        # json args
        self.args['color'] = self._handle_boolean_args('color')
        self._convert_strings_to_list('groups')
        self._convert_strings_to_list('users')
        self._convert_strings_to_list('escrow_users')
        self._validate_combinatorial_args()
        self._validate_args()

        verify_on_load = self.args['subparser_name'] in ['listrecipients', 'import', 'interpreter']

        # Build the list of recipients that this command will act on
        self._build_recipient_list()

        # If there are defined repositories of keys and certificates, load them
        if not self.iddbcached:
            self.identities.cabundle = self.args['cabundle']
            self.identities.load_certs_from_directory(
                self.args['certpath'],
                verify_on_load=verify_on_load,
                connectmap=connectmap)
            self.identities.load_keys_from_directory(self.args['keypath'])
            self._validate_identities()

        if 'pwname' in self.args and self.args['pwname']:
            self._resolve_directory_path()
        self.args['card_slot'] = self.args['card_slot'] if self.args['card_slot'] else 0
        if 'nopassphrase' in self.selected_args and not self.args['nopassphrase']:
            if self.args['verbosity'] != -1:
                print_card_info(self.args['card_slot'],
                                self.identities.iddb[self.args['identity']],
                                self.args['verbosity'],
                                self.args['color'],
                                self.args['theme_map'])
            self.passphrase = getpass.getpass("Enter Pin/Passphrase: ")

        ####################################################################
    def _resolve_directory_path(self):
        """This handles how a user inputs the pwname, this tries to be smart
        good luck everybody else"""
        ####################################################################
        pwd = os.getcwd()
        pwd_pwname = os.path.normpath(os.path.join(pwd, self.args['pwname']))
        if self.args['pwstore'] in pwd_pwname:
            self.args['pwname'] = pwd_pwname.replace(self.args['pwstore'] + os.sep, '')

        ##################################################################
    def safety_check(self):
        """ This provides a sanity check that you are the owner of a password."""
        ##################################################################
        try:
            password = PasswordEntry()
            password.read_password_data(os.path.join(self.args['pwstore'], self.args['pwname']))
            return (self.args['identity'] in password['recipients'].keys(), password['metadata']['creator'])
        except PasswordIOError:
            return (True, None)

        ##################################################################
    def update_pass(self, pass_value):
        """Fully updated a password record"""
        ##################################################################
        pass_entry = PasswordEntry()
        pass_entry.read_password_data(os.path.join(self.args['pwstore'], self.args['pwname']))
        swap_pass = PasswordEntry()
        swap_pass.add_recipients(secret=pass_value,
                                 distributor=self.args['identity'],
                                 recipients=[self.args['identity']],
                                 identitydb=self.identities,
                                 passphrase=self.passphrase,
                                 card_slot=self.args['card_slot'],
                                 pwstore=self.args['pwstore']
                                )
        pass_entry['recipients'][self.args['identity']] = swap_pass['recipients'][self.args['identity']]
        pass_entry.write_password_data(os.path.join(self.args['pwstore'], self.args['pwname']),
                                       overwrite=self.args['overwrite'])

        ##################################################################
    def create_pass(self, password1, description, authorizer, recipient_list=None):
        """ This writes password data to a file."""
        ##################################################################
        password_metadata = {}
        password_metadata['description'] = description
        password_metadata['authorizer'] = authorizer
        password_metadata['creator'] = self.args['identity']
        password_metadata['name'] = self.args['pwname']
        if self.args['noescrow']:
            self.args['min_escrow'] = None
            self.args['escrow_users'] = None
        if recipient_list is None:
            recipient_list = [self.args['identity']]

        password = PasswordEntry(**password_metadata)

        password.add_recipients(secret=password1,
                                distributor=self.args['identity'],
                                recipients=recipient_list,
                                identitydb=self.identities,
                                passphrase=self.passphrase,
                                card_slot=self.args['card_slot'],
                                escrow_users=self.args['escrow_users'],
                                minimum=self.args['min_escrow'],
                                pwstore=self.args['pwstore']
                               )

        password.write_password_data(os.path.join(self.args['pwstore'], self.args['pwname']),
                                     overwrite=self.args['overwrite'])

        ##################################################################
    def create_or_update_pass(self, password1, description, authorizer, recipient_list=None):
        """ This creates new or updates existing passwords """
        ##################################################################
        safe, owner = self.safety_check()
        if safe or self.args['overwrite']:
            if owner and owner != self.args['identity']:
                self.update_pass(password1)
            else:
                self.create_pass(password1, description, authorizer, recipient_list)
        else:
            raise NotThePasswordOwnerError(self.args['identity'], owner, self.args['pwname'])

        ##################################################################
    def delete_pass(self):
        """This deletes a password that the user has created, useful for testing"""
        ##################################################################
        filepath = os.path.join(self.args['pwstore'], self.args['pwname'])
        try:
            os.remove(filepath)
        except OSError:
            raise PasswordIOError("Password '%s' not found" % self.args['pwname'])

        ##################################################################
    def rename_pass(self):
        """This renames a password that the user has created"""
        ##################################################################
        oldpath = os.path.join(self.args['pwstore'], self.args['pwname'])
        newpath = os.path.join(self.args['pwstore'], self.args['rename'])
        try:
            os.rename(oldpath, newpath)
            password = PasswordEntry()
            password.read_password_data(newpath)
            password['metadata']['name'] = self.args['rename']
            password.write_password_data(newpath)

        except OSError:
            raise PasswordIOError("Password '%s' not found" % self.args['pwname'])

        ##################################################################
    def _run_command_execution(self):
        """ Passes the argparse Namespace object of parsed arguments   """
        ##################################################################
        raise NotImplementedError

        ##################################################################
    def _build_recipient_list(self):
        """take groups and users and make a SOA for the recipients"""
        ##################################################################
        try:
            if 'groups' in self.args and self.args['groups']:
                self.recipient_list += self._parse_group_membership()
            if 'users' in self.args and self.args['users']:
                self.recipient_list += self.args['users']
            self.recipient_list = [x.strip() for x in list(set(self.recipient_list))]
            self.escrow_and_recipient_list = self.recipient_list + self.args['escrow_users']
            for user in self.escrow_and_recipient_list:
                if str(user) == '':
                    raise NullRecipientError
        except KeyError:  # If this is a command with no users, don't worry about it
            pass

        ##################################################################
    def _parse_group_membership(self):
        """Concat membership of supplied groups"""
        ##################################################################
        member_list = []
        try:
            for group in self.args['groups']:
                member_list += [user.strip() for user in self.args[group.strip()].split(",") if user.strip()]
            return member_list
        except KeyError as err:
            raise GroupDefinitionError(str(err))

        ##################################################################
    def _get_config_args(self, config, cli_args):
        """Return the configuration from the config file"""
        ##################################################################
        try:
            with open(config, 'r') as fname:
                config_args = yaml.safe_load(fname)
            if config_args is None:
                config_args = {}
            return config_args
        except IOError:
            if cli_args['verbosity'] != -1:
                print("INFO: No .pkpassrc file found")
            return {}
        except yaml.parser.ParserError:
            raise ConfigParseError("Parsing error with config file, please check syntax")

        ##################################################################
    def _validate_args(self):
        ##################################################################
        raise NotImplementedError

        ##################################################################
    def _validate_combinatorial_args(self):
        """ This is a weird function name so: combinatorial in this case
            means that one of the 'combinatorial' arguments are required
            however, not all of them are necessarily required.
            Ex: We need certs, we can get this from certpath or connect
            we do not need both of these arguments but at least one is
            required"""
        ##################################################################
        # we want a multi-dim of lists, this way if more combinations come up
        # that would be required in a 1 or more capacity, we just add
        # a list to this list
        args_list = [['certpath', 'connect'], ['certpath', 'keypath']]
        for arg_set in args_list:
            valid = False
            for arg in arg_set:
                if arg in self.args and self.args[arg] is not None:
                    valid = True
                    break
            if not valid:
                raise CliArgumentError(
                    "'%s' or '%s' is required" % tuple(arg_set))

        ##################################################################
    def _parse_json_arguments(self, argument):
        """ Parses the json.loads arguments as dictionaries to use"""
        ##################################################################
        try:
            if argument in self.args and self.args[argument]:
                if isinstance(self.args[argument], dict):
                    return self.args[argument]
                return json.loads(self.args[argument])
            return None
        except ValueError as err:
            raise JsonArgumentError(argument, err)

        ##################################################################
    def _validate_identities(self, swap_list=None):
        """Ensure identities meet criteria for processing"""
        ##################################################################
        if not swap_list:
            swap_list = self.escrow_and_recipient_list
        for recipient in swap_list:
            self.identities.verify_identity(recipient)
            if recipient not in self.identities.iddb.keys():
                raise CliArgumentError(
                    "Error: Recipient '%s' is not in the recipient database" %
                    recipient)

        if self.args['identity'] in self.identities.iddb.keys():
            self.identities.verify_identity(self.args['identity'])
        else:
            raise CliArgumentError(
                "Error: Your user '%s' is not in the recipient database" %
                self.args['identity'])

        ##################################################################
    def _print_debug(self):
        ##################################################################
        print(self.recipient_list)
        print(self.identities.iddb.keys())

        ##################################################################
    def color_print(self, string, color_type):
        """Handle the color printing for objects"""
        ##################################################################
        return util.color_prepare(string,
                                  color_type,
                                  self.args['color'],
                                  self.args['theme_map'])

        ##################################################################
    def progress_bar(self, value, endvalue, bar_length=50):
        """Allow a command to print off a progress bar"""
        ##################################################################
        percent = float(value) / endvalue
        arrow = '-' * int(round(percent * bar_length)-1) + '>'
        spaces = ' ' * (bar_length - len(arrow))
        sys.stdout.write("\rPercent: [{0}] {1}%".format(arrow + spaces, int(round(percent * 100))))
        sys.stdout.flush()



#!/usr/bin/env python
"""This module tests the show module"""

from __future__ import absolute_import
import unittest
import argparse
import mock
import libpkpass.commands.cli as cli
import libpkpass.commands.show as show
from libpkpass.errors import DecryptionError, CliArgumentError
from .basetest.basetest import CONFIG, BADPIN

class ShowErrors(unittest.TestCase):
    """This class tests the show class"""

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='show', identity='r1',
                                                nopassphrase="true",
                                                all=None,
                                                pwname='test',
                                                config=CONFIG))
    def test_decryption_error(self, subparser_name):
        """test decryption functionality"""
        ret = False
        try:
            show.Show(cli.Cli())
        except DecryptionError as error:
            if error.msg == BADPIN:
                ret = True
        self.assertTrue(ret)

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='show', identity='bleh',
                                                nopassphrase="true",
                                                all=None,
                                                pwname='test',
                                                config=CONFIG))
    def test_recipient_not_in_database(self, subparser_name):
        """test decryption functionality"""
        ret = False
        try:
            show.Show(cli.Cli())
        except CliArgumentError as error:
            if error.msg == "Error: Your user 'bleh' is not in the recipient database":
                ret = True
        self.assertTrue(ret)

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='show', identity='r1',
                                                nopassphrase="true",
                                                all=True,
                                                pwname='*test*',
                                                config=CONFIG))
    def test_showall_decryption_error(self, subparser_name):
        """test decryption functionality"""
        ret = False
        try:
            show.Show(cli.Cli())
        except DecryptionError as error:
            if error.msg == BADPIN:
                ret = True
        self.assertTrue(ret)

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='show', identity='r1',
                                                nopassphrase="true",
                                                all=None,
                                                config=CONFIG))
    def test_show_nopass_error(self, subparser_name):
        """test decryption functionality"""
        ret = False
        try:
            show.Show(cli.Cli())
        except KeyError as error:
            if str(error) == "'pwname'":
                ret = str(error)
        self.assertEqual(ret, "'pwname'")


if __name__ == '__main__':
    unittest.main()
